/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.openoffice.sdk.awt;

import com.sun.star.awt.DeviceInfo;
import com.sun.star.awt.PosSize;
import com.sun.star.awt.Rectangle;
import com.sun.star.awt.WindowAttribute;
import com.sun.star.awt.WindowClass;
import com.sun.star.awt.WindowDescriptor;
import com.sun.star.awt.XDevice;
import com.sun.star.awt.XSystemDependentWindowPeer;
import com.sun.star.awt.XToolkit;
import com.sun.star.awt.XTopWindow;
import com.sun.star.awt.XVclWindowPeer;
import com.sun.star.awt.XWindow;
import com.sun.star.awt.XWindow2;
import com.sun.star.awt.XWindowPeer;
import com.sun.star.beans.PropertyValue;
import com.sun.star.beans.XPropertySet;
import com.sun.star.comp.helper.Bootstrap;
import com.sun.star.container.XIndexAccess;
import com.sun.star.frame.FrameSearchFlag;
import com.sun.star.frame.XComponentLoader;
import com.sun.star.frame.XController;
import com.sun.star.frame.XDesktop;
import com.sun.star.frame.XFrame;
import com.sun.star.frame.XFrames;
import com.sun.star.frame.XFramesSupplier;
import com.sun.star.frame.XLayoutManager;
import com.sun.star.frame.XModel;
import com.sun.star.lang.XComponent;
import com.sun.star.uno.AnyConverter;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;

/**
 *
 * @author ariel
 */
public class FrameLoader {
    
    private final XComponentContext m_xContext;
    private XToolkit m_xToolkit;
    
    public FrameLoader( XComponentContext xContext ){
        m_xContext = xContext;
        
        try {
            m_xToolkit = (XToolkit) UnoRuntime.queryInterface(
                    XToolkit.class,
                    xContext.getServiceManager().createInstanceWithContext(
                    "com.sun.star.awt.Toolkit", xContext));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    
    protected XComponent loadComponent(String sURL, boolean bHasLayoutManager, Rectangle aWinSize){
        XComponent xComponent = null;
        try {
            // in order to center the window on the screen
            // we get first the default display work area
            // We could use Java AWT
            //      java.awt.Toolkit.getDefaultToolkit().getScreenSize()
            // but to make this portable, use
            // the UNPUBLISHED/UNDOCUMENTED com.sun.star.awt.DisplayAccess
            Rectangle aWorkArea = getDefaultDisplayWorkArea();
            
            // the window size
            if (aWinSize == null) {
                aWinSize = aWorkArea.Width > 0 && aWorkArea.Height > 0
                        ? new Rectangle(0, 0,
                        (int) (aWorkArea.Width / 1.5),
                        (int) (aWorkArea.Height / 1.5))
                        : new Rectangle(0, 0, 400, 300);
            }

            // calculate the window position, in the center of the work area
            calcCenterPosSize(aWinSize, aWorkArea);
            
            // create a top window
            XWindowPeer xWindowPeer = createTopWindow(null, aWinSize);
            XWindow2 xWindow = (XWindow2) UnoRuntime.queryInterface(
                    XWindow2.class, xWindowPeer);
            
            //*****************************************************************
            
            // create a Frame
            XFrame xFrame = (XFrame) UnoRuntime.queryInterface(
                    XFrame.class, 
                    m_xContext.getServiceManager().createInstanceWithContext(
                    "com.sun.star.frame.Frame", m_xContext));            
            
            // initialize the frame with our window
            xFrame.initialize(xWindow);
            
            
            //*****************************************************************
            // do we want our frame to be part of the desktop hierarchy
            // and so take part of the application framework?
            XDesktop xDesktop = (XDesktop) UnoRuntime.queryInterface(
                    XDesktop.class,
                    m_xContext.getServiceManager().createInstanceWithContext(
                    "com.sun.star.frame.Desktop", m_xContext));
            
            // if NOT, don't append it!
            // But we should, among other things, add an 
            // XTerminateListener at the XDesktop, 
            // an XTopWindowListener at our window, etc.
            // [An example: if our component is modified and 
            // the user tries to close it, the user won't be warned with 
            // the Save/Discard/Cancel dialog
            // So in short: we are by ourselves!]
            XFramesSupplier xFramesSupplier = (XFramesSupplier) UnoRuntime.queryInterface(
                    XFramesSupplier.class, xDesktop);
            
            XFrames xFrames = xFramesSupplier.getFrames();
            
            // comment/un-comment to test what happens...
            xFrames.append(xFrame);
            
            //*****************************************************************
            
            // get rid of the LayoutManager
            if (!bHasLayoutManager) {
                XPropertySet xFramePropertySet = (XPropertySet) UnoRuntime.queryInterface(
                        XPropertySet.class, xFrame);
                XLayoutManager xLayoutManager = null;

                // in our case, the Title should be set as a window property!
                //xFramePropertySet.setPropertyValue("Title", "Frame loader demo");
                xFramePropertySet.setPropertyValue("LayoutManager", xLayoutManager);
            }
            
            //*****************************************************************
            
            // load the component
            // as loading will inmediatly set the window visible,
            // we should load the component hidden if we want to perform
            // some tasks before showing the component to the user
            PropertyValue[] aMediaDescriptor = new PropertyValue[1];
            /*aMediaDescriptor[0] = new PropertyValue();
            aMediaDescriptor[0].Name = "Hidden";
            aMediaDescriptor[0].Value = Boolean.TRUE;*/
            
            
            XComponentLoader xComponentLoader = (XComponentLoader) UnoRuntime.queryInterface(
                    XComponentLoader.class, xFrame);
            
            // set both target name and search flag 
            // to load the component in[side] itself
            xComponent = xComponentLoader.loadComponentFromURL(
                    sURL, "_self", FrameSearchFlag.SELF, aMediaDescriptor);
            
            //*****************************************************************
            
            // unfortunatlly, the PosSize set when created the window
            // will have no efect, so we must set it again
            // once the window *is* *visible*
            
            // the following CANNOT be false
            XWindow xContainerWindow = xFrame.getContainerWindow();            
             boolean bAreSame = UnoRuntime.areSame(xContainerWindow, xWindow);
            System.out.printf("XFrame.getContainerWindow() returns our window? %b\n", bAreSame );                        

            // if the window was not visible ...
            // xWindow/xFrame.getContainerWindow().setVisible(true);
            xWindow.setPosSize(
                                aWinSize.X, aWinSize.X, 
                                aWinSize.Width, aWinSize.Height, 
                                PosSize.POSSIZE);
            
            // set the Title
            XVclWindowPeer xVclWindowPeer = (XVclWindowPeer) UnoRuntime.queryInterface(
                    XVclWindowPeer.class, xWindow);
            xVclWindowPeer.setProperty("Title", "Frame loader demo");
            
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            return xComponent;
        }
    }
    
    
    private XWindowPeer createTopWindow(XWindowPeer xParentWindowPeer, Rectangle aPosSize) {
        XWindowPeer xTopWindow = null;
        try {
            if (m_xToolkit != null) {
                WindowDescriptor aWindowDescriptor = new WindowDescriptor();

                aWindowDescriptor.Bounds = aPosSize;
                aWindowDescriptor.Parent = xParentWindowPeer;
                aWindowDescriptor.ParentIndex = -1;
                aWindowDescriptor.Type = WindowClass.TOP;
                aWindowDescriptor.WindowAttributes =
                        WindowAttribute.BORDER |
                        WindowAttribute.CLOSEABLE |
                        /*WindowAttribute.FULLSIZE |*/
                        WindowAttribute.MOVEABLE |
                        WindowAttribute.SIZEABLE;
                aWindowDescriptor.WindowServiceName = "workwindow";
                
                xTopWindow = m_xToolkit.createWindow(aWindowDescriptor);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            return xTopWindow;
        }
    }
    
    public Rectangle getDefaultDisplayWorkArea() {
        Rectangle aWorkArea = new Rectangle();
        try {
            XPropertySet xDisplayAccess = (XPropertySet) UnoRuntime.queryInterface(
                    XPropertySet.class, m_xContext.getServiceManager().
                        createInstanceWithContext(
                        "com.sun.star.awt.DisplayAccess", m_xContext));
            XIndexAccess xDisplayIndexAccess = (XIndexAccess) UnoRuntime.queryInterface(
                    XIndexAccess.class, xDisplayAccess);
            
            int nDefaultDisplay = -1;
            try {
                nDefaultDisplay = AnyConverter.toInt(
                        xDisplayAccess.getPropertyValue("DefaultDisplay"));
            } catch (Exception e) {
                e.printStackTrace();
            }            
            
            XPropertySet xDefaultDisplay = (XPropertySet) UnoRuntime.queryInterface(
                    XPropertySet.class,
                    xDisplayIndexAccess.getByIndex(nDefaultDisplay) );            
            
            aWorkArea = (Rectangle) AnyConverter.toObject(
                    Rectangle.class,
                    xDefaultDisplay.getPropertyValue("WorkArea"));
            
        } catch (Exception e) {
            e.printStackTrace();
        }
        return aWorkArea;
    }
    
    private void calcCenterPosSize(
                                    Rectangle aWinPosSize /* IN-OUT */, 
                                    Rectangle aParentWinPosSize) {
        try {

            int aParentWinHeight = aParentWinPosSize.Height;
            int aParentWinWidth = aParentWinPosSize.Width;

            int aWinWidth = aWinPosSize.Width;
            int aWinHeight = aWinPosSize.Height;

            aWinPosSize.X = ((aParentWinWidth / 2) - (aWinWidth / 2));
            aWinPosSize.Y = ((aParentWinHeight / 2) - (aWinHeight / 2));

        } catch (Exception e) {
            e.printStackTrace();
        }
    }



    private static void displaySreenShot(java.awt.Rectangle aRect) {
        try {
            java.awt.Robot aRobot = new java.awt.Robot();
            java.awt.image.BufferedImage aImage = aRobot.createScreenCapture(aRect);

            javax.swing.JFrame aFrame = new javax.swing.JFrame("Screenshot");
            aFrame.setDefaultCloseOperation(javax.swing.JFrame.DISPOSE_ON_CLOSE);

            javax.swing.JLabel aLabel = new javax.swing.JLabel(
                    new javax.swing.ImageIcon(aImage));

            aFrame.getContentPane().add(aLabel);
            aFrame.setSize(/*aLabel.getPreferredSize()*/aRect.width, aRect.height);
            aFrame.setLocation(0, 0);
            aFrame.setVisible(true);

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public static void main(String[] args) {
        
        XComponentContext xContext = null;
        try {
            xContext = Bootstrap.bootstrap();
            
            FrameLoader demo = new FrameLoader(xContext);
            
            String sURL = "private:factory/swriter";
            XComponent xComponent = demo.loadComponent(sURL, true, null);
            
            if ( xComponent != null ){                

                XModel xModel = (XModel) UnoRuntime.queryInterface(
                        XModel.class, xComponent);
                XController xController = xModel.getCurrentController();
                XFrame xFrame = xController.getFrame();
                XWindow xContainerWin = xFrame.getContainerWindow();
                Rectangle aPosSize = xContainerWin.getPosSize();

                XDevice xDevice = (XDevice) UnoRuntime.queryInterface(XDevice.class, xContainerWin);
                DeviceInfo aDeviceInfo = xDevice.getInfo();

                java.awt.Rectangle aRect = new java.awt.Rectangle(
                        aPosSize.X, aPosSize.Y,
                        aPosSize.Width, aPosSize.Height);

                // Bring the window to the front
                // WARNING: does NOT work!
                XTopWindow xTopWindow = (XTopWindow) UnoRuntime.queryInterface(
                        XTopWindow.class, xContainerWin);
                xTopWindow.toFront();

                // you have to bring it manually!
                // so, put to sleep...
                //Thread.sleep(1000);

                // This takes a screentshot of the contanier window (without the title bar)
                //displaySreenShot( aRect );

                Thread.sleep(1000);

                // This takes a screentshot of the contanier window
                // (hopefully with the title bar... one should add 1 or 2 pixels)
                // WARNING: it does not take into account the window border
                //          and may be it misses other hints fro the window manager
                displaySreenShot( new java.awt.Rectangle(
                        aPosSize.X , aPosSize.Y - aDeviceInfo.TopInset,
                        aPosSize.Width, aPosSize.Height + aDeviceInfo.TopInset) );

                //*************************************************************

                System.out.println("\nPress 'Return' finish the example.");
                java.io.BufferedReader reader = new java.io.BufferedReader(
                        new java.io.InputStreamReader(System.in));
                reader.read();

                //*************************************************************

            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            System.exit(0);
        }
    }

}
